import { Frequency } from '@modules/ad/core/domain/ad.types';
import { PublishMessageWhenAdIsCreatedDomainEventHandler } from '@modules/ad/core/application/event-handlers/publish-message-when-ad-is-created.domain-event-handler';
import { AdCreatedDomainEvent } from '@modules/ad/core/domain/events/ad-created.domain-events';
import { Test, TestingModule } from '@nestjs/testing';
import { MESSAGE_PUBLISHER } from '@src/app.constants';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('Publish message when ad is created domain event handler', () => {
  let publishMessageWhenAdIsCreatedDomainEventHandler: PublishMessageWhenAdIsCreatedDomainEventHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        PublishMessageWhenAdIsCreatedDomainEventHandler,
      ],
    }).compile();

    publishMessageWhenAdIsCreatedDomainEventHandler =
      module.get<PublishMessageWhenAdIsCreatedDomainEventHandler>(
        PublishMessageWhenAdIsCreatedDomainEventHandler,
      );
  });

  it('should publish a message', () => {
    jest.spyOn(mockMessagePublisher, 'publish');
    const adCreatedDomainEvent: AdCreatedDomainEvent = {
      id: 'some-domain-event-id',
      aggregateId: 'some-aggregate-id',
      userId: 'some-user-id',
      driver: false,
      passenger: true,
      frequency: Frequency.PUNCTUAL,
      fromDate: '2023-06-28',
      toDate: '2023-06-28',
      monTime: undefined,
      tueTime: undefined,
      wedTime: '07:15',
      thuTime: undefined,
      friTime: undefined,
      satTime: undefined,
      sunTime: undefined,
      monMarginDuration: 900,
      tueMarginDuration: 900,
      wedMarginDuration: 900,
      thuMarginDuration: 900,
      friMarginDuration: 900,
      satMarginDuration: 900,
      sunMarginDuration: 900,
      seatsProposed: 3,
      seatsRequested: 1,
      strict: false,
      waypoints: [
        {
          position: 0,
          houseNumber: '5',
          street: 'Avenue Foch',
          locality: 'Nancy',
          postalCode: '54000',
          country: 'France',
          lat: 48.689445,
          lon: 6.1765102,
        },
        {
          position: 1,
          locality: 'Paris',
          postalCode: '75000',
          country: 'France',
          lat: 48.8566,
          lon: 2.3522,
        },
      ],
      metadata: {
        timestamp: new Date('2023-06-28T05:00:00Z').getTime(),
        correlationId: 'some-correlation-id',
      },
    };
    publishMessageWhenAdIsCreatedDomainEventHandler.handle(
      adCreatedDomainEvent,
    );
    expect(publishMessageWhenAdIsCreatedDomainEventHandler).toBeDefined();
    expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
      'ad.created',
      '{"id":"some-domain-event-id","aggregateId":"some-aggregate-id","userId":"some-user-id","driver":false,"passenger":true,"frequency":"PUNCTUAL","fromDate":"2023-06-28","toDate":"2023-06-28","wedTime":"07:15","monMarginDuration":900,"tueMarginDuration":900,"wedMarginDuration":900,"thuMarginDuration":900,"friMarginDuration":900,"satMarginDuration":900,"sunMarginDuration":900,"seatsProposed":3,"seatsRequested":1,"strict":false,"waypoints":[{"position":0,"houseNumber":"5","street":"Avenue Foch","locality":"Nancy","postalCode":"54000","country":"France","lat":48.689445,"lon":6.1765102},{"position":1,"locality":"Paris","postalCode":"75000","country":"France","lat":48.8566,"lon":2.3522}],"metadata":{"timestamp":1687928400000,"correlationId":"some-correlation-id"}}',
    );
  });
});
