import { Module, Provider } from '@nestjs/common';
import { HealthHttpController } from './interface/http-controllers/health.http.controller';
import { TerminusModule } from '@nestjs/terminus';
import { MESSAGE_PUBLISHER } from 'src/app.constants';
import { RepositoriesHealthIndicatorUseCase } from './core/application/usecases/repositories.health-indicator.usecase';
import { AdRepository } from '../ad/infrastructure/ad.repository';
import { AD_REPOSITORY } from './health.di-tokens';
import { HealthGrpcController } from './interface/grpc-controllers/health.grpc.controller';
import { AdModule } from '@modules/ad/ad.module';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';

const grpcControllers = [HealthGrpcController];

const httpControllers = [HealthHttpController];

const useCases: Provider[] = [RepositoriesHealthIndicatorUseCase];

const repositories: Provider[] = [
  {
    provide: AD_REPOSITORY,
    useClass: AdRepository,
  },
];

const messageBrokers: Provider[] = [
  {
    provide: MESSAGE_PUBLISHER,
    useClass: MessageBrokerPublisher,
  },
];

@Module({
  imports: [TerminusModule, AdModule],
  controllers: [...grpcControllers, ...httpControllers],
  providers: [...useCases, ...repositories, ...messageBrokers],
})
export class HealthModule {}
